# Generated by Django 3.2.13 on 2022-06-03 13:07

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0019_merge_20211026_0919'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('industry', '0003_auto_20220601_1747'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserCharacters',
            fields=[
                ('character_ownership', models.OneToOneField(help_text='ownership of this character on Auth', on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='industry_character', serialize=False, to='authentication.characterownership')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User x Characters',
                'verbose_name_plural': 'User x Characters',
                'ordering': ['user'],
            },
        ),
    ]



from django.test import TestCase
from django.urls import reverse as r


#################
# injecting jobs
# job_details = dict()
# job_details['is_corp_job'] = False
# job_details['job_id'] = 123456
#
# a = EveEntity.objects.get_or_create_esi(id=56749)[0]
# job_details['blueprint_name'] = a.name
# job_details['blueprint_id'] = a.id
#
# job_details['activity_id'] = _get_activity_by_id(2)
#
# job_details['duration'] = 3600 * 2
# job_details['start_date'] = '2023-04-27T00:00:00Z'
# job_details['end_date'] = '2023-04-29T00:00:00Z'
# job_details['status'] = 'active'
# job_details['installer_id'] = user_id
# job_details['installer_portrait_url_32'] = character_portrait_url(user_id, 32)
# job_details['installer_portrait_url_64'] = character_portrait_url(user_id, 64)
# job_details['installer_portrait_url_128'] = character_portrait_url(user_id, 128)
# job_details['installer_portrait_url_256'] = character_portrait_url(user_id, 256)
# job_details['facility_id'] = '1041037509462'
#
# station = _get_structure(_request_headers, '1041037509462')
# if not station:
#     logger.error('error getting station')
#     job_details['station_name'] = ''
# else:
#     job_details['station_name'] = station.name
#
# return_list = list()
# return_list.append(job_details)
#
# return return_list
# end of injecting jobs


class ViewTest(TestCase):
    def setUp(self):
        self.resp = self.client.get(r('industry:index'))

    def test_index_requires_login(self):
        """
        GET /industry must require login
        """
        self.assertRedirects(self.resp, f'/account/login/?next={r("industry:index")}')




from django.contrib import admin

from .models import Facility, UserCharacters


@admin.register(Facility)
class FacilityAdmin(admin.ModelAdmin):
    list_display = ['name', 'facility_id', 'solar_system', 'owner_id']


@admin.register(UserCharacters)
class UserCharactersAdmin(admin.ModelAdmin):
    list_display = ['user', 'character_ownership']



# from django.conf import settings

# put your app settings here, like this
# (And don't forget to uncomment the import above):

# EXAMPLE_SETTING_ONE = getattr(settings, "EXAMPLE_SETTING_ONE", None)



# Generated by Django 3.2.13 on 2022-06-01 17:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('eveuniverse', '0007_evetype_description'),
        ('industry', '0002_delete_usertoken'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='facility',
            name='solar_system_id',
        ),
        migrations.AddField(
            model_name='facility',
            name='solar_system',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    to='eveuniverse.evesolarsystem'),
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-01 17:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('industry', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='UserToken',
        ),
    ]



import logging

from allianceauth.eveonline.models import EveCharacter
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, redirect

from esi.decorators import token_required
from esi.models import Token
from esi.clients import esi_client_factory

from allianceauth.authentication.models import CharacterOwnership
from .models import UserCharacters

from .helper import _get_personal_jobs, _get_corp_jobs, _process_jobs

logger = logging.getLogger(__name__)

_scopes = [
    'esi-industry.read_character_jobs.v1',
    'esi-industry.read_corporation_jobs.v1',
    'esi-universe.read_structures.v1',
]

cache_facility_id = {}
cache_character_id = {}


@login_required
@permission_required("industry.view_industry")
def index(request):
    # return redirect('industry:list_jobs', character_id=request.user.profile.main_character.character_id)
    characters = list()

    tokens = Token.objects.filter(user__pk=request.user.pk).require_scopes(_scopes).require_valid()
    if tokens:
        for u in UserCharacters.objects.filter(user=request.user):
            characters.append(u.character_ownership.character)

    if len(characters) == 1:
        return redirect('industry:list_jobs', character_id=request.user.profile.main_character.character_id)

    context = {
        "characters": characters
    }
    return render(request, "industry/registered_characters.html", context)


@login_required
@permission_required("industry.view_industry")
@token_required(scopes=_scopes)
def char_selector(request, t):
    token_char = EveCharacter.objects.get(character_id=t.character_id)
    character_ownership = CharacterOwnership.objects.select_related(
        "character"
    ).get(user=request.user, character=token_char)

    if character_ownership:
        uc = UserCharacters.objects.filter(character_ownership=character_ownership)
        if not uc:
            UserCharacters.objects.create(
                character_ownership=character_ownership,
                user=request.user
            )

    return redirect('industry:list_jobs', character_id=t.character_id)


@login_required
@permission_required("industry.view_industry")
def list_jobs(request, character_id):
    corp_id = request.user.profile.main_character.corporation_id

    t = Token.objects.filter(user__pk=request.user.pk, character_id=character_id)\
        .require_scopes(_scopes).require_valid().first()
    if not t:
        return redirect('industry:selector')

    _request_headers = {
        'accept': 'application/json',
        'Cache-Control': 'no-cache',
        'authorization': 'Bearer ' + t.valid_access_token()
    }

    jobs = _get_personal_jobs(t.character_id, _request_headers)
    _processed_jobs = _process_jobs(_request_headers, jobs)

    jobs = _get_corp_jobs(corp_id, _request_headers)
    _corp_jobs = _process_jobs(_request_headers, jobs, True)

    if _corp_jobs:
        _processed_jobs += _corp_jobs

    character_data = {
        'character_name': t.character_name,
        'character_id': t.character_id,
        'token_id': t.pk
    }

    context = {
        "items": _processed_jobs,
        "character_data": character_data,
        # "user_tokens": CharacterOwnership.objects.filter(user=request.user).all(),
        "user_tokens": UserCharacters.objects.filter(user=request.user).all(),
    }

    return render(request, "industry/index.html", context)



# Generated by Django 3.2.13 on 2022-05-27 16:43

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('esi', '0011_add_token_indices'),
    ]

    operations = [
        migrations.CreateModel(
            name='Facility',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('facility_id', models.BigIntegerField(verbose_name='Facility')),
                ('name', models.CharField(max_length=2048, verbose_name='Name')),
                ('owner_id', models.BigIntegerField(verbose_name='Owner Id')),
                ('solar_system_id', models.BigIntegerField(verbose_name='Solar System Id')),
                ('type_id', models.BigIntegerField(verbose_name='Type')),
            ],
            options={
                'verbose_name': 'Facility',
                'verbose_name_plural': 'Facilities',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Industry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='UserToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('character_id', models.IntegerField(db_index=True, help_text='The ID of the EVE character who authenticated by SSO.')),
                ('token', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='esi.token')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Industry User x Token',
                'verbose_name_plural': 'Industry User x Tokens',
                'ordering': ['user'],
            },
        ),
    ]



import logging

from allianceauth.eveonline.evelinks.eveimageserver import character_portrait_url
from eveuniverse.models import EveEntity, EveSolarSystem
from .models import Facility

from datetime import datetime, timedelta
import json
import requests

logger = logging.getLogger(__name__)

cache_character_id = {}
cache_corporation_id = {}


def _process_jobs(_request_headers, jobs, is_corp: bool = False) -> list:
    _processed = list()
    cache_character_id.clear()
    cache_corporation_id.clear()

    for j in jobs:
        job_details = dict()
        job_details['is_corp_job'] = is_corp
        job_details['job_id'] = j.get('job_id')

        a = EveEntity.objects.get_or_create_esi(id=j.get('blueprint_type_id'))[0]
        job_details['blueprint_name'] = a.name
        job_details['blueprint_id'] = a.id

        job_details['activity_id'] = _get_activity_by_id(j.get('activity_id'))

        job_details['duration'] = _secondsToTime(j.get('duration'))
        job_details['start_date'] = _fromStrToDate(j.get('start_date'))
        job_details['end_date'] = _fromStrToDate(j.get('end_date'))
        job_details['status'] = j.get('status')
        job_details['installer_id'] = j.get('installer_id')
        job_details['installer_portrait_url_32'] = character_portrait_url(j.get('installer_id'), 32)
        job_details['installer_portrait_url_64'] = character_portrait_url(j.get('installer_id'), 64)
        job_details['installer_portrait_url_128'] = character_portrait_url(j.get('installer_id'), 128)
        job_details['installer_portrait_url_256'] = character_portrait_url(j.get('installer_id'), 256)

        station = _get_structure(_request_headers, j.get('facility_id'))
        if not station:
            logger.error('error getting station')
            job_details['station_name'] = ''
        else:
            job_details['station_name'] = station.name

        if is_corp:
            job_character = _get_character(_request_headers, job_details['installer_id'])
            job_details['installer_name'] = job_character['name']
            job_details['installer_corp'] = job_character['corporation_id']
            job_details['installer_corp_name'] = job_character['corporation'].get('description')

        _processed.append(job_details)

    cache_character_id.clear()
    cache_corporation_id.clear()

    return _processed


def _get_activity_by_id(activity_id: int) -> str:
    activities = {
        1: "Manufacturing",
        2: "Researching Technology",
        3: "Time Efficiency Research",
        4: "Material Efficiency Research",
        5: "Copying",
        6: "Duplicating",
        7: "Reverse Engineering",
        8: "Invention",
        9: "Reaction",
    }

    return activities[activity_id] if activity_id in activities.keys() else activity_id


def _get_structure(_request_headers: dict, facility_id: str) -> Facility:
    try:
        facility = Facility.objects.filter(facility_id=facility_id).first()

        if facility and facility.solar_system is not None:
            return facility
        else:
            facility = Facility()

        r = requests.get(
            f'https://esi.evetech.net/latest/universe/structures/{facility_id}/?datasource=tranquility',
            headers=_request_headers
        )
        if r.status_code == 200:
            station = json.loads(r.content)

            _solar_system = EveSolarSystem.objects.get_or_create(id=station['solar_system_id'])

            if len(_solar_system) > 0:
                solar_system = _solar_system[0]
            else:
                solar_system = _solar_system

            facility.facility_id = facility_id
            facility.name = station['name']
            facility.owner_id = station['owner_id']
            facility.solar_system = solar_system
            facility.type_id = station['type_id']
            facility.save()

            return facility

    except Exception as exxx:
        logger.error(exxx)
        return None


def _get_character(_request_headers: dict, character_id: str) -> dict:
    try:
        if character_id in cache_character_id.keys():
            return cache_character_id.get(character_id)

        # get character name
        r = requests.get(
            f'https://esi.evetech.net/latest/characters/{character_id}/?datasource=tranquility',
            headers=_request_headers
        )
        if r.status_code == 200:
            character = json.loads(r.content)
            corporation = _get_corporation_name(_request_headers, character['corporation_id'])
            character['corporation'] = corporation
            cache_character_id[character_id] = character
            return character

        return None

    except Exception as ex:
        logger.error(f'error getting character name by id => {ex}')
        return None


def _get_corporation_name(_request_headers, corporation_id: int) -> dict:

    try:
        if corporation_id in cache_corporation_id.keys():
            return cache_corporation_id.get(corporation_id)

        # get character name
        r = requests.get(
            f'https://esi.evetech.net/latest/corporations/{corporation_id}/?datasource=tranquility',
            headers=_request_headers
        )
        if r.status_code == 200:
            corporation = json.loads(r.content)
            cache_corporation_id[corporation_id] = corporation
            return corporation

        return None

    except Exception as ex:
        logger.error(f'error getting character name by id => {ex}')
        return None


def _get_personal_jobs(user_id: int, _request_headers: dict, completed: bool = False) -> list:
    _url = f'https://esi.evetech.net/latest/characters/{user_id}/industry/jobs/?datasource=tranquility'

    if completed:
        _url = _url + '&include_completed=true'

    return _get_jobs(_url, _request_headers)


def _get_corp_jobs(corp_id: int, _request_headers: dict, completed: bool = False) -> list:
    _url = f'https://esi.evetech.net/latest/corporations/{corp_id}/industry/jobs/?datasource=tranquility'

    if completed:
        _url = _url + '&include_completed=true'

    return _get_jobs(_url, _request_headers)


def _get_jobs(url: str, _request_headers: dict) -> list:
    try:
        r = requests.get(url, headers=_request_headers)

        if r.status_code == 200:
            return json.loads(r.content)
    except requests.exceptions.ConnectionError:
        logger.error("connection failed")
        return []
    else:
        logger.error(f'requests error {r.status_code}')
        return []


def _secondsToTime(seconds: int) -> str:
    return str(timedelta(seconds=seconds))


def _fromStrToDate(date_time_str: str) -> datetime:
    time_obj = datetime.strptime(date_time_str, '%Y-%m-%dT%H:%M:%SZ')
    return time_obj



from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter
from django.db import models
from django.contrib.auth import get_user_model

from django.utils.translation import gettext as _
from eveuniverse.models import EveSolarSystem

User = get_user_model()


class Industry(models.Model):
    pass


class UserCharacters(models.Model):
    user = models.ForeignKey(User, blank=False, on_delete=models.CASCADE)
    character_ownership = models.OneToOneField(
        CharacterOwnership,
        related_name="industry_character",
        on_delete=models.CASCADE,
        primary_key=True,
        help_text="ownership of this character on Auth",
    )

    def __str__(self):
        return f'{self.user.profile.main_character} - {self.character_ownership.character}'

    class Meta:
        ordering = ['user']
        verbose_name = _('User x Characters')
        verbose_name_plural = _('User x Characters')


class Facility(models.Model):
    facility_id = models.BigIntegerField(_('Facility'), null=False, blank=False)
    name = models.CharField(_('Name'), null=False, blank=False, max_length=2048)
    owner_id = models.BigIntegerField(_('Owner Id'), null=False, blank=False)
    solar_system = models.ForeignKey(EveSolarSystem, null=True, blank=True, on_delete=models.CASCADE)
    type_id = models.BigIntegerField(_('Type'), null=False, blank=False)

    class Meta:
        ordering = ['name']
        verbose_name = _('Facility')
        verbose_name_plural = _('Facilities')



from django.urls import path

from . import views

app_name = "industry"

urlpatterns = [
    path("", views.index, name="index"),
    path("select", views.char_selector, name="selector"),
    path("jobs/<character_id>", views.list_jobs, name="list_jobs"),
    # path("delete/<pk>/", views.UserTokenDelete.as_view(), name="character-delete"),
]



from celery import shared_task

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)

# Create your tasks here


# Example Task:
# @shared_task
# def my_task():
#     pass



from django.template import (Context, Template, Node, TemplateSyntaxError, Variable, Library)

register = Library()

TEMPLATE = """
<img src="https://images.evetech.net/characters/{{ id }}/portrait?size={{ size }}">
"""


@register.tag(name='character_portrait')
def do_character_portrait(parser, token):
    try:
        # if not passed, append default portrait size
        if len(token.split_contents()) < 3:
            token.contents += ' 32'
        tag_name, _id, _size = token.split_contents()
    except ValueError:
        raise TemplateSyntaxError("%r tag requires 1 argument" % token.contents.split()[0])
    return CharacterPortraitNode(_id, _size)


class CharacterPortraitNode(Node):
    def __init__(self, _id, _size):
        self.id = Variable(_id)
        self.size = Variable(_size)

    def render(self, context):
        _values = {
            'id':  self.id.resolve(context),
            'size': self.size.resolve(context)
        }

        t = Template(TEMPLATE)
        c = Context(_values)
        return t.render(c)


# register = Library()
# register.tag('character_portrait', do_character_portrait)
